import abc
import typing

from .... import exceptions, platforms, utils


class PandaMachineDef(metaclass=abc.ABCMeta):
    """Container class for Unicorn architecture-specific definitions"""

    @property
    @abc.abstractmethod
    def arch(self) -> platforms.Architecture:
        """The architecture ID"""
        raise NotImplementedError("This is an abstract method.")

    @property
    @abc.abstractmethod
    def byteorder(self) -> platforms.Byteorder:
        """The byte order"""
        raise NotImplementedError("This is an abstract method.")

    @property
    @abc.abstractmethod
    def panda_arch(self) -> str:
        """The panda architecture to use"""
        raise NotImplementedError("This is an abstract method.")

    _registers: typing.Dict[str, typing.Optional[str]] = {}

    def panda_reg(self, name: str) -> str:
        if name in self._registers:
            res = self._registers[name]
            if res is None:
                raise exceptions.UnsupportedRegisterError(
                    f"Register {name} not recognized by Panda for {self.arch}:{self.byteorder}"
                )
            return res
        else:
            raise ValueError(
                f"Unknown register for {self.arch}:{self.byteorder}: {name}"
            )

    def check_panda_reg(self, name: str) -> bool:
        """Convert a register name to panda cpu field, index, mask

        This must cover all names defined in the CPU state model
        for this arch/mode/byteorder, or return 0,
        which always indicates an invalid register
        """
        if name in self._registers and self._registers[name] is not None:
            return True
        else:
            return False

    @classmethod
    def for_platform(cls, platform: platforms.Platform):
        """Find the appropriate MachineDef for your architecture

        Arguments:
            arch: The architecture ID you want
            mode: The mode ID you want
            byteorder: The byteorderness you want

        Returns:
            An instance of the appropriate MachineDef

        Raises:
            ValueError: If no MachineDef subclass matches your request
        """
        try:
            return utils.find_subclass(
                cls,
                lambda x: x.arch == platform.architecture
                and x.byteorder == platform.byteorder,
            )
        except:
            raise ValueError(
                f"No machine model for {platform.architecture}:{platform.byteorder}"
            )



import angr

from .fixups import FixupMemoryMixin
from .memtrack import TrackerMemoryMixin


class DefaultMemoryPlugin(  # type: ignore[misc]
    TrackerMemoryMixin, FixupMemoryMixin, angr.storage.DefaultMemory
):
    pass



import abc

from ... import utils
from ...platforms import ABI, Architecture, Byteorder, Platform


class ErrnoResolver(metaclass=abc.ABCMeta):
    """Errno resolver

    For some UNGODLY REASON,
    the System V ABIs can't agree on errno values across platforms.
    """

    @property
    @abc.abstractmethod
    def platform(self) -> Platform:
        """The platform this errno resolver supports"""
        raise NotImplementedError()

    @property
    @abc.abstractmethod
    def abi(self) -> ABI:
        """The ABI this errno resolver supports"""
        raise NotImplementedError()

    # These are the numbers from asm-generic.
    # MOST architectures use these... (MIPS!!!)
    errno_to_label = {
        1: "EPERM",
        2: "ENOENT",
        3: "ESRCH",
        4: "EINTR",
        5: "EIO",
        6: "ENXIO",
        7: "E2BIG",
        8: "ENOEXEC",
        9: "EBADF",
        10: "ECHILD",
        11: "EAGAIN",
        12: "ENOMEM",
        13: "EACCES",
        14: "EFAULT",
        15: "ENOTBLK",
        16: "EBUSY",
        17: "EEXIST",
        18: "EXDEV",
        19: "ENODEV",
        20: "ENOTDIR",
        21: "EISDIR",
        22: "EINVAL",
        23: "ENFILE",
        24: "EMFILE",
        25: "ENOTTY",
        26: "ETXTBSY",
        27: "EFBIG",
        28: "ENOSPC",
        29: "ESPIPE",
        30: "EROFS",
        31: "EMLINK",
        32: "EPIPE",
        33: "EDOM",
        34: "ERANGE",
        35: "EDEADLK",
        36: "ENAMETOOLONG",
        37: "ENOLCK",
        38: "ENOSYS",
        39: "ENOTEMPTY",
        40: "ELOOP",
        42: "ENOMSG",
        43: "EIDRM",
        44: "ECHRNG",
        45: "EL2NSYNC",
        46: "EL3HLT",
        47: "EL3RST",
        48: "ELNRNG",
        49: "EUNATCH",
        50: "ENOCSI",
        51: "EL2HLT",
        52: "EBADE",
        53: "EBADR",
        54: "EXFULL",
        55: "ENOANO",
        56: "EBADRQC",
        57: "EBADSLT",
        59: "EBFONT",
        60: "ENOSTR",
        61: "ENODATA",
        62: "ETIME",
        63: "ENOSR",
        64: "ENONET",
        65: "ENOPKG",
        66: "EREMOTE",
        67: "ENOLINK",
        68: "EADV",
        69: "ESRMNT",
        70: "ECOMM",
        71: "EPROTO",
        72: "EMULTIHOP",
        73: "EDOTDOT",
        74: "EBADMSG",
        75: "EOVERFLOW",
        76: "ENOTUNIQ",
        77: "EBADFD",
        78: "EREMCHG",
        79: "ELIBACC",
        80: "ELIBBAD",
        81: "ELIBSCN",
        82: "ELIBMAX",
        83: "ELIBEXEC",
        84: "EILSEQ",
        85: "ERESTART",
        86: "ESTRPIPE",
        87: "EUSERS",
        88: "ENOTSOCK",
        89: "EDESTADDRREQ",
        90: "EMSGSIZE",
        91: "EPROTOTYPE",
        92: "ENOPROTOOPT",
        93: "EPROTONOSUPPORT",
        94: "ESOCKTNOSUPPORT",
        95: "EOPNOTSUPP",
        96: "EPFNOSUPPORT",
        97: "EAFNOSUPPORT",
        98: "EADDRINUSE",
        99: "EADDRNOTAVAIL",
        100: "ENETDOWN",
        101: "ENETUNREACH",
        102: "ENETRESET",
        103: "ECONNABORTED",
        104: "ECONNRESET",
        105: "ENOBUFS",
        106: "EISCONN",
        107: "ENOTCONN",
        108: "ESHUTDOWN",
        109: "ETOOMANYREFS",
        110: "ETIMEDOUT",
        111: "ECONNREFUSED",
        112: "EHOSTDOWN",
        113: "EHOSTUNREACH",
        114: "EALREADY",
        115: "EINPROGRESS",
        116: "ESTALE",
        117: "EUCLEAN",
        118: "ENOTNAM",
        119: "ENAVAIL",
        120: "EISNAM",
        121: "EREMOTEIO",
        122: "EDQUOT",
        123: "ENOMEDIUM",
        124: "EMEDIUMTYPE",
        125: "ECANCELED",
        126: "ENOKEY",
        127: "EKEYEXPIRED",
        128: "EKEYREVOKED",
        129: "EKEYREJECTED",
        130: "EOWNERDEAD",
        131: "ENOTRECOVERABLE",
        132: "ERFKILL",
        133: "EHWPOISON",
    }

    label_to_description = {
        "EPERM": "Operation not permitted",
        "ENOENT": "No such file or directory",
        "ESRCH": "No such process",
        "EINTR": "Interrupted system call",
        "EIO": "Input/output error",
        "ENXIO": "No such device or address",
        "E2BIG": "Argument list too long",
        "ENOEXEC": "Exec format error",
        "EBADF": "Bad file descriptor",
        "ECHILD": "No child processes",
        "EAGAIN": "Resource temporarily unavailable",
        "ENOMEM": "Cannot allocate memory",
        "EACCES": "Permission denied",
        "EFAULT": "Bad address",
        "ENOTBLK": "Block device required",
        "EBUSY": "Device or resource busy",
        "EEXIST": "File exists",
        "EXDEV": "Invalid cross-device link",
        "ENODEV": "No such device",
        "ENOTDIR": "Not a directory",
        "EISDIR": "Is a directory",
        "EINVAL": "Invalid argument",
        "ENFILE": "Too many open files in system",
        "EMFILE": "Too many open files",
        "ENOTTY": "Inappropriate ioctl for device",
        "ETXTBSY": "Text file busy",
        "EFBIG": "File too large",
        "ENOSPC": "No space left on device",
        "ESPIPE": "Illegal seek",
        "EROFS": "Read-only file system",
        "EMLINK": "Too many links",
        "EPIPE": "Broken pipe",
        "EDOM": "Numerical argument out of domain",
        "ERANGE": "Numerical result out of range",
        "EDEADLK": "Resource deadlock avoided",
        "ENAMETOOLONG": "File name too long",
        "ENOLCK": "No locks available",
        "ENOSYS": "Function not implemented",
        "ENOTEMPTY": "Directory not empty",
        "ELOOP": "Too many levels of symbolic links",
        "EWOULDBLOCK": "Resource temporarily unavailable",
        "ENOMSG": "No message of desired type",
        "EIDRM": "Identifier removed",
        "ECHRNG": "Channel number out of range",
        "EL2NSYNC": "Level 2 not synchronized",
        "EL3HLT": "Level 3 halted",
        "EL3RST": "Level 3 reset",
        "ELNRNG": "Link number out of range",
        "EUNATCH": "Protocol driver not attached",
        "ENOCSI": "No CSI structure available",
        "EL2HLT": "Level 2 halted",
        "EBADE": "Invalid exchange",
        "EBADR": "Invalid request descriptor",
        "EXFULL": "Exchange full",
        "ENOANO": "No anode",
        "EBADRQC": "Invalid request code",
        "EBADSLT": "Invalid slot",
        "EDEADLCK": "Resource deadlock avoided",
        "EDEADLOCK": "Resource deadlock avoided",
        "EBFONT": "Bad font file format",
        "ENOSTR": "Device not a stream",
        "ENODATA": "No data available",
        "ETIME": "Timer expired",
        "ENOSR": "Out of streams resources",
        "ENONET": "Machine is not on the network",
        "ENOPKG": "Package not installed",
        "EREMOTE": "Object is remote",
        "ENOLINK": "Link has been severed",
        "EADV": "Advertise error",
        "ESRMNT": "Srmount error",
        "ECOMM": "Communication error on send",
        "EPROTO": "Protocol error",
        "EMULTIHOP": "Multihop attempted",
        "EDOTDOT": "RFS specific error",
        "EBADMSG": "Bad message",
        "EOVERFLOW": "Value too large for defined data type",
        "ENOTUNIQ": "Name not unique on network",
        "EBADFD": "File descriptor in bad state",
        "EREMCHG": "Remote address changed",
        "ELIBACC": "Can not access a needed shared library",
        "ELIBBAD": "Accessing a corrupted shared library",
        "ELIBSCN": ".lib section in a.out corrupted",
        "ELIBMAX": "Attempting to link in too many shared libraries",
        "ELIBEXEC": "Cannot exec a shared library directly",
        "EILSEQ": "Invalid or incomplete multibyte or wide character",
        "ERESTART": "Interrupted system call should be restarted",
        "ESTRPIPE": "Streams pipe error",
        "EUSERS": "Too many users",
        "ENOTSOCK": "Socket operation on non-socket",
        "EDESTADDRREQ": "Destination address required",
        "EMSGSIZE": "Message too long",
        "EPROTOTYPE": "Protocol wrong type for socket",
        "ENOPROTOOPT": "Protocol not available",
        "EPROTONOSUPPORT": "Protocol not supported",
        "ESOCKTNOSUPPORT": "Socket type not supported",
        "EOPNOTSUPP": "Operation not supported",
        "EPFNOSUPPORT": "Protocol family not supported",
        "EAFNOSUPPORT": "Address family not supported by protocol",
        "EADDRINUSE": "Address already in use",
        "EADDRNOTAVAIL": "Cannot assign requested address",
        "ENETDOWN": "Network is down",
        "ENETUNREACH": "Network is unreachable",
        "ENETRESET": "Network dropped connection on reset",
        "ECONNABORTED": "Software caused connection abort",
        "ECONNRESET": "Connection reset by peer",
        "ENOBUFS": "No buffer space available",
        "EISCONN": "Transport endpoint is already connected",
        "ENOTCONN": "Transport endpoint is not connected",
        "ESHUTDOWN": "Cannot send after transport endpoint shutdown",
        "ETOOMANYREFS": "Too many references: cannot splice",
        "ETIMEDOUT": "Connection timed out",
        "ECONNREFUSED": "Connection refused",
        "EHOSTDOWN": "Host is down",
        "EHOSTUNREACH": "No route to host",
        "EALREADY": "Operation already in progress",
        "EINPROGRESS": "Operation now in progress",
        "ESTALE": "Stale file handle",
        "EUCLEAN": "Structure needs cleaning",
        "ENOTNAM": "Not a XENIX named type file",
        "ENAVAIL": "No XENIX semaphores available",
        "EISNAM": "Is a named type file",
        "EREMOTEIO": "Remote I/O error",
        "EDQUOT": "Disk quota exceeded",
        "ENOMEDIUM": "No medium found",
        "EMEDIUMTYPE": "Wrong medium type",
        "ECANCELED": "Operation canceled",
        "ENOKEY": "Required key not available",
        "EKEYEXPIRED": "Key has expired",
        "EKEYREVOKED": "Key has been revoked",
        "EKEYREJECTED": "Key was rejected by service",
        "EOWNERDEAD": "Owner died",
        "ENOTRECOVERABLE": "State not recoverable",
        "ERFKILL": "Operation not possible due to RF-kill",
        "EHWPOISON": "Memory page has hardware error",
        "ENOTSUP": "Operation not supported",
    }

    def get_label(self, errno: int) -> str:
        """Get the label for an errno by its number

        NOTE: Some numbers are tied to mulitple errors.
        This function will always return the same label
        for the same number.  No guarantees
        that it's the one you want in case of collision.

        This will raise an exception in case of an unknown number

        Arguments:
            errno: The error number

        Returns:
            A string containing the macro defined for that number
        """

        if errno not in self.errno_to_label:
            raise KeyError(f"Unknown errno {errno}")
        return self.errno_to_label[errno]

    def get_description(self, errno: int) -> str:
        """Get the descriptive text for an errno by its number

        NOTE: Some numbers are tied to mulitple errors.
        This function will always return the same description
        for the same number.  No guarantees
        that it's the one you want in case of collision.

        This will return the string "Unknown error {errno}"
        in case of an unknown number.

        Arguments:
            errno: The error number

        Returns:
            The descriptive text for this errno.
        """
        try:
            label = self.get_label(errno)
            if label not in self.label_to_description:
                return f"Unknown error {errno}"
            return self.label_to_description[label]
        except KeyError:
            return f"Unknown error {errno}"

    @classmethod
    def for_platform(cls, platform: Platform, abi: ABI) -> "ErrnoResolver":
        """Instantiate an errno resolver by platform and ABI.

        Arguments:
            platform: The platform for which this resolver is defined
            abi: The ABI for which this resolver is defined

        Returns:
            The errno resolver for this platform and ABI
        """
        try:
            return utils.find_subclass(
                cls, lambda x: x.platform == platform and x.abi == abi
            )
        except ValueError:
            raise ValueError(f"no errno resolver for {platform} with ABI '{abi}'")


class AArch64ErrnoResolver(ErrnoResolver):
    platform = Platform(Architecture.AARCH64, Byteorder.LITTLE)
    abi = ABI.SYSTEMV
    # aarch64 uses generic errno


class AMD64ErrnoResolver(ErrnoResolver):
    platform = Platform(Architecture.X86_64, Byteorder.LITTLE)
    abi = ABI.SYSTEMV
    # amd64 uses generic errno


class ArmELErrnoResolver(ErrnoResolver):
    platform = Platform(Architecture.ARM_V5T, Byteorder.LITTLE)
    abi = ABI.SYSTEMV
    # armel uses generic errno


class ArmHFErrnoResolver(ErrnoResolver):
    platform = Platform(Architecture.ARM_V7A, Byteorder.LITTLE)
    abi = ABI.SYSTEMV
    # armhf uses generic errno


class I386ErrnoResolver(ErrnoResolver):
    platform = Platform(Architecture.X86_32, Byteorder.LITTLE)
    abi = ABI.SYSTEMV
    # i386 uses generic errno


class LoongArch64ErrnoResolver(ErrnoResolver):
    platform = Platform(Architecture.LOONGARCH64, Byteorder.LITTLE)
    abi = ABI.SYSTEMV
    # la64 uses generic errno


class MIPSErrnoResolver(ErrnoResolver):
    abi = ABI.SYSTEMV
    # MIPS Absolutely doesn't use generic errno
    errno_to_label = {
        1: "EPERM",
        2: "ENOENT",
        3: "ESRCH",
        4: "EINTR",
        5: "EIO",
        6: "ENXIO",
        7: "E2BIG",
        8: "ENOEXEC",
        9: "EBADF",
        10: "ECHILD",
        11: "EAGAIN",
        12: "ENOMEM",
        13: "EACCES",
        14: "EFAULT",
        15: "ENOTBLK",
        16: "EBUSY",
        17: "EEXIST",
        18: "EXDEV",
        19: "ENODEV",
        20: "ENOTDIR",
        21: "EISDIR",
        22: "EINVAL",
        23: "ENFILE",
        24: "EMFILE",
        25: "ENOTTY",
        26: "ETXTBSY",
        27: "EFBIG",
        28: "ENOSPC",
        29: "ESPIPE",
        30: "EROFS",
        31: "EMLINK",
        32: "EPIPE",
        33: "EDOM",
        34: "ERANGE",
        35: "ENOMSG",
        36: "EIDRM",
        37: "ECHRNG",
        38: "EL2NSYNC",
        39: "EL3HLT",
        40: "EL3RST",
        41: "ELNRNG",
        42: "EUNATCH",
        43: "ENOCSI",
        44: "EL2HLT",
        45: "EDEADLK",
        46: "ENOLCK",
        50: "EBADE",
        51: "EBADR",
        52: "EXFULL",
        53: "ENOANO",
        54: "EBADRQC",
        55: "EBADSLT",
        56: "EDEADLOCK",
        59: "EBFONT",
        60: "ENOSTR",
        61: "ENODATA",
        62: "ETIME",
        63: "ENOSR",
        64: "ENONET",
        65: "ENOPKG",
        66: "EREMOTE",
        67: "ENOLINK",
        68: "EADV",
        69: "ESRMNT",
        70: "ECOMM",
        71: "EPROTO",
        73: "EDOTDOT",
        74: "EMULTIHOP",
        77: "EBADMSG",
        78: "ENAMETOOLONG",
        79: "EOVERFLOW",
        80: "ENOTUNIQ",
        81: "EBADFD",
        82: "EREMCHG",
        83: "ELIBACC",
        84: "ELIBBAD",
        85: "ELIBSCN",
        86: "ELIBMAX",
        87: "ELIBEXEC",
        88: "EILSEQ",
        89: "ENOSYS",
        90: "ELOOP",
        91: "ERESTART",
        92: "ESTRPIPE",
        93: "ENOTEMPTY",
        94: "EUSERS",
        95: "ENOTSOCK",
        96: "EDESTADDRREQ",
        97: "EMSGSIZE",
        98: "EPROTOTYPE",
        99: "ENOPROTOOPT",
        120: "EPROTONOSUPPORT",
        121: "ESOCKTNOSUPPORT",
        122: "EOPNOTSUPP",
        123: "EPFNOSUPPORT",
        124: "EAFNOSUPPORT",
        125: "EADDRINUSE",
        126: "EADDRNOTAVAIL",
        127: "ENETDOWN",
        128: "ENETUNREACH",
        129: "ENETRESET",
        130: "ECONNABORTED",
        131: "ECONNRESET",
        132: "ENOBUFS",
        133: "EISCONN",
        134: "ENOTCONN",
        135: "EUCLEAN",
        137: "ENOTNAM",
        138: "ENAVAIL",
        139: "EISNAM",
        140: "EREMOTEIO",
        141: "EINIT",
        142: "EREMDEV",
        143: "ESHUTDOWN",
        144: "ETOOMANYREFS",
        145: "ETIMEDOUT",
        146: "ECONNREFUSED",
        147: "EHOSTDOWN",
        148: "EHOSTUNREACH",
        149: "EALREADY",
        150: "EINPROGRESS",
        151: "ESTALE",
        158: "ECANCELED",
        159: "ENOMEDIUM",
        160: "EMEDIUMTYPE",
        161: "ENOKEY",
        162: "EKEYEXPIRED",
        163: "EKEYREVOKED",
        164: "EKEYREJECTED",
        165: "EOWNERDEAD",
        166: "ENOTRECOVERABLE",
        167: "ERFKILL",
        168: "EHWPOISON",
        1133: "EDQUOT",
    }


class MIPSBEErrnoResolver(MIPSErrnoResolver):
    platform = Platform(Architecture.MIPS32, Byteorder.BIG)
    # mips uses the MIPS errno list


class MIPSELErrnoResolver(MIPSErrnoResolver):
    platform = Platform(Architecture.MIPS32, Byteorder.LITTLE)
    # mipsel uses the MIPS errno list


class MIPS64BEErrnoResolver(MIPSErrnoResolver):
    platform = Platform(Architecture.MIPS64, Byteorder.BIG)
    # mips64 uses the MIPS errno list


class MIPS64ELErrnoResolver(MIPSErrnoResolver):
    platform = Platform(Architecture.MIPS64, Byteorder.LITTLE)
    # mips64el uses the MIPS errno list


class PowerPCErrnoResolver(ErrnoResolver):
    platform = Platform(Architecture.POWERPC32, Byteorder.BIG)
    abi = ABI.SYSTEMV

    # The ppc errno list is almost identical to generic,
    # except that EDEADLOCK is different.
    #
    # In generic, it's been aliased to EDEADLK,
    # although the ordering of the header suggests that at one time
    # it looked like it does here.
    errno_to_label = ErrnoResolver.errno_to_label | {35: "EDEADLCK", 58: "EDEADLOCK"}


class RiscV64ErrnoResolver(ErrnoResolver):
    platform = Platform(Architecture.RISCV64, Byteorder.LITTLE)
    abi = ABI.SYSTEMV
    # riscv64 uses the generic errno list


__all__ = ["ErrnoResolver"]



from .aarch64 import AArch64ElfRelocator
from .amd64 import AMD64ElfRelocator
from .arm import (
    Armv5TElfRelocator,
    Armv6MElfRelocator,
    Armv7AElfRelocator,
    Armv7MElfRelocator,
    Armv7RElfRelocator,
)
from .i386 import I386ElfRelocator
from .loongarch import LoongArch64ElfRelocator
from .mips import MIPSELElfRelocator, MIPSElfRelocator
from .ppc import PowerPCElfRelocator
from .rela import ElfRelocator
from .riscv64 import RISCV64ElfRelocator
from .xtensa import XtensaElfRelocator

__all__ = [
    "AArch64ElfRelocator",
    "AMD64ElfRelocator",
    "Armv5TElfRelocator",
    "Armv6MElfRelocator",
    "Armv7AElfRelocator",
    "Armv7MElfRelocator",
    "Armv7RElfRelocator",
    "I386ElfRelocator",
    "LoongArch64ElfRelocator",
    "MIPSElfRelocator",
    "MIPSELElfRelocator",
    "PowerPCElfRelocator",
    "RISCV64ElfRelocator",
    "XtensaElfRelocator",
    "ElfRelocator",
]



import unicorn

from ....platforms import Architecture, Byteorder
from .machdef import UnicornMachineDef


class ARMMachineDef(UnicornMachineDef):
    """Base Unicorn machine definition for 32-bit ARM"""

    uc_arch = unicorn.UC_ARCH_ARM
    uc_mode = unicorn.UC_MODE_ARM

    def __init__(self):
        self._registers = {
            "r0": unicorn.arm_const.UC_ARM_REG_R0,
            "r1": unicorn.arm_const.UC_ARM_REG_R1,
            "r2": unicorn.arm_const.UC_ARM_REG_R2,
            "r3": unicorn.arm_const.UC_ARM_REG_R3,
            "r4": unicorn.arm_const.UC_ARM_REG_R4,
            "r5": unicorn.arm_const.UC_ARM_REG_R5,
            "r6": unicorn.arm_const.UC_ARM_REG_R6,
            "r7": unicorn.arm_const.UC_ARM_REG_R7,
            "r8": unicorn.arm_const.UC_ARM_REG_R8,
            # r9 doubles as the Static base pointer
            "r9": unicorn.arm_const.UC_ARM_REG_R9,
            "sb": unicorn.arm_const.UC_ARM_REG_SB,
            # r10 doubles as the Stack Limit pointer
            "r10": unicorn.arm_const.UC_ARM_REG_R10,
            "sl": unicorn.arm_const.UC_ARM_REG_SL,
            # r11 doubles as the Frame Pointer, if desired.
            "r11": unicorn.arm_const.UC_ARM_REG_R11,
            "fp": unicorn.arm_const.UC_ARM_REG_FP,
            # r12 doubles as the Intra-call scratch register
            "r12": unicorn.arm_const.UC_ARM_REG_R12,
            "ip": unicorn.arm_const.UC_ARM_REG_IP,
            "sp": unicorn.arm_const.UC_ARM_REG_SP,
            "lr": unicorn.arm_const.UC_ARM_REG_LR,
            "pc": unicorn.arm_const.UC_ARM_REG_PC,
        }


class ARMMachineMixinM:
    """Mixin for ARM M-series machine models"""

    def __init__(self):
        super().__init__()
        self._registers.update(
            {
                # NOTE: PSR is aliased to CPSR
                # This is an artifact of the fact that Unicorn
                # seems to emulate a mash-up of M- and A-series arm.
                "psr": unicorn.arm_const.UC_ARM_REG_CPSR,
                "primask": unicorn.arm_const.UC_ARM_REG_PRIMASK,
                "basepri": unicorn.arm_const.UC_ARM_REG_BASEPRI,
                "faultmask": unicorn.arm_const.UC_ARM_REG_FAULTMASK,
                "control": unicorn.arm_const.UC_ARM_REG_CONTROL,
                "msp": unicorn.arm_const.UC_ARM_REG_MSP,
                "psp": unicorn.arm_const.UC_ARM_REG_PSP,
            }
        )


class ARMMachineMixinRA:
    """Mixin for ARM R- or A- series machine models"""

    def __init__(self):
        super().__init__()
        self._registers.update(
            {
                "cpsr": unicorn.arm_const.UC_ARM_REG_CPSR,
                "spsr": unicorn.arm_const.UC_ARM_REG_SPSR,
                # NOTE: None of the banked registers have Unicorn IDs
                "sp_usr": unicorn.arm_const.UC_ARM_REG_INVALID,
                "lr_usr": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r8_usr": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r9_usr": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r10_usr": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r11_usr": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r12_usr": unicorn.arm_const.UC_ARM_REG_INVALID,
                "sp_hyp": unicorn.arm_const.UC_ARM_REG_INVALID,
                "spsr_hyp": unicorn.arm_const.UC_ARM_REG_INVALID,
                "elr_hyp": unicorn.arm_const.UC_ARM_REG_INVALID,
                "sp_svc": unicorn.arm_const.UC_ARM_REG_INVALID,
                "lr_svc": unicorn.arm_const.UC_ARM_REG_INVALID,
                "spsr_svc": unicorn.arm_const.UC_ARM_REG_INVALID,
                "sp_abt": unicorn.arm_const.UC_ARM_REG_INVALID,
                "lr_abt": unicorn.arm_const.UC_ARM_REG_INVALID,
                "spsr_abt": unicorn.arm_const.UC_ARM_REG_INVALID,
                "sp_und": unicorn.arm_const.UC_ARM_REG_INVALID,
                "lr_und": unicorn.arm_const.UC_ARM_REG_INVALID,
                "spsr_und": unicorn.arm_const.UC_ARM_REG_INVALID,
                "sp_mon": unicorn.arm_const.UC_ARM_REG_INVALID,
                "lr_mon": unicorn.arm_const.UC_ARM_REG_INVALID,
                "spsr_mon": unicorn.arm_const.UC_ARM_REG_INVALID,
                "sp_irq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "lr_irq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "spsr_irq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "sp_fiq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "lr_fiq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "spsr_fiq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r8_fiq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r9_fiq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r10_fiq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r11_fiq": unicorn.arm_const.UC_ARM_REG_INVALID,
                "r12_fiq": unicorn.arm_const.UC_ARM_REG_INVALID,
            }
        )


class ARMMachineMixinFP:
    """Mixin for ARM machine models with basic FPUs"""

    def __init__(self):
        super().__init__()
        self._registers.update(
            {
                "fpscr": unicorn.arm_const.UC_ARM_REG_FPSCR,
                "fpexc": unicorn.arm_const.UC_ARM_REG_FPEXC,
                "fpsid": unicorn.arm_const.UC_ARM_REG_FPSID,
                "mvfr0": unicorn.arm_const.UC_ARM_REG_INVALID,
                "mvfr1": unicorn.arm_const.UC_ARM_REG_INVALID,
                "d0": unicorn.arm_const.UC_ARM_REG_D0,
                "s0": unicorn.arm_const.UC_ARM_REG_S0,
                "s1": unicorn.arm_const.UC_ARM_REG_S1,
                "d1": unicorn.arm_const.UC_ARM_REG_D1,
                "s2": unicorn.arm_const.UC_ARM_REG_S2,
                "s3": unicorn.arm_const.UC_ARM_REG_S3,
                "d2": unicorn.arm_const.UC_ARM_REG_D2,
                "s4": unicorn.arm_const.UC_ARM_REG_S4,
                "s5": unicorn.arm_const.UC_ARM_REG_S5,
                "d3": unicorn.arm_const.UC_ARM_REG_D3,
                "s6": unicorn.arm_const.UC_ARM_REG_S6,
                "s7": unicorn.arm_const.UC_ARM_REG_S7,
                "d4": unicorn.arm_const.UC_ARM_REG_D4,
                "s8": unicorn.arm_const.UC_ARM_REG_S8,
                "s9": unicorn.arm_const.UC_ARM_REG_S9,
                "d5": unicorn.arm_const.UC_ARM_REG_D5,
                "s10": unicorn.arm_const.UC_ARM_REG_S10,
                "s11": unicorn.arm_const.UC_ARM_REG_S11,
                "d6": unicorn.arm_const.UC_ARM_REG_D6,
                "s12": unicorn.arm_const.UC_ARM_REG_S12,
                "s13": unicorn.arm_const.UC_ARM_REG_S13,
                "d7": unicorn.arm_const.UC_ARM_REG_D7,
                "s14": unicorn.arm_const.UC_ARM_REG_S14,
                "s15": unicorn.arm_const.UC_ARM_REG_S15,
                "d8": unicorn.arm_const.UC_ARM_REG_D8,
                "s16": unicorn.arm_const.UC_ARM_REG_S16,
                "s17": unicorn.arm_const.UC_ARM_REG_S17,
                "d9": unicorn.arm_const.UC_ARM_REG_D9,
                "s18": unicorn.arm_const.UC_ARM_REG_S18,
                "s19": unicorn.arm_const.UC_ARM_REG_S19,
                "d10": unicorn.arm_const.UC_ARM_REG_D10,
                "s20": unicorn.arm_const.UC_ARM_REG_S20,
                "s21": unicorn.arm_const.UC_ARM_REG_S21,
                "d11": unicorn.arm_const.UC_ARM_REG_D11,
                "s22": unicorn.arm_const.UC_ARM_REG_S22,
                "s23": unicorn.arm_const.UC_ARM_REG_S23,
                "d12": unicorn.arm_const.UC_ARM_REG_D12,
                "s24": unicorn.arm_const.UC_ARM_REG_S24,
                "s25": unicorn.arm_const.UC_ARM_REG_S25,
                "d13": unicorn.arm_const.UC_ARM_REG_D13,
                "s26": unicorn.arm_const.UC_ARM_REG_S26,
                "s27": unicorn.arm_const.UC_ARM_REG_S27,
                "d14": unicorn.arm_const.UC_ARM_REG_D14,
                "s28": unicorn.arm_const.UC_ARM_REG_S28,
                "s29": unicorn.arm_const.UC_ARM_REG_S29,
                "d15": unicorn.arm_const.UC_ARM_REG_D15,
                "s30": unicorn.arm_const.UC_ARM_REG_S30,
                "s31": unicorn.arm_const.UC_ARM_REG_S31,
            }
        )


class ARMMachineMixinVFP:
    """Mixin for ARM machine models with VFP/NEON support"""

    def __init__(self):
        super().__init__()
        self._registers.update(
            {
                "fpscr": unicorn.arm_const.UC_ARM_REG_FPSCR,
                "fpexc": unicorn.arm_const.UC_ARM_REG_FPEXC,
                "fpsid": unicorn.arm_const.UC_ARM_REG_FPSID,
                "mvfr0": unicorn.arm_const.UC_ARM_REG_INVALID,
                "mvfr1": unicorn.arm_const.UC_ARM_REG_INVALID,
                "q0": unicorn.arm_const.UC_ARM_REG_Q0,
                "d0": unicorn.arm_const.UC_ARM_REG_D0,
                "s0": unicorn.arm_const.UC_ARM_REG_S0,
                "s1": unicorn.arm_const.UC_ARM_REG_S1,
                "d1": unicorn.arm_const.UC_ARM_REG_D1,
                "s2": unicorn.arm_const.UC_ARM_REG_S2,
                "s3": unicorn.arm_const.UC_ARM_REG_S3,
                "q1": unicorn.arm_const.UC_ARM_REG_Q1,
                "d2": unicorn.arm_const.UC_ARM_REG_D2,
                "s4": unicorn.arm_const.UC_ARM_REG_S4,
                "s5": unicorn.arm_const.UC_ARM_REG_S5,
                "d3": unicorn.arm_const.UC_ARM_REG_D3,
                "s6": unicorn.arm_const.UC_ARM_REG_S6,
                "s7": unicorn.arm_const.UC_ARM_REG_S7,
                "q2": unicorn.arm_const.UC_ARM_REG_Q2,
                "d4": unicorn.arm_const.UC_ARM_REG_D4,
                "s8": unicorn.arm_const.UC_ARM_REG_S8,
                "s9": unicorn.arm_const.UC_ARM_REG_S9,
                "d5": unicorn.arm_const.UC_ARM_REG_D5,
                "s10": unicorn.arm_const.UC_ARM_REG_S10,
                "s11": unicorn.arm_const.UC_ARM_REG_S11,
                "q3": unicorn.arm_const.UC_ARM_REG_Q3,
                "d6": unicorn.arm_const.UC_ARM_REG_D6,
                "s12": unicorn.arm_const.UC_ARM_REG_S12,
                "s13": unicorn.arm_const.UC_ARM_REG_S13,
                "d7": unicorn.arm_const.UC_ARM_REG_D7,
                "s14": unicorn.arm_const.UC_ARM_REG_S14,
                "s15": unicorn.arm_const.UC_ARM_REG_S15,
                "q4": unicorn.arm_const.UC_ARM_REG_Q4,
                "d8": unicorn.arm_const.UC_ARM_REG_D8,
                "s16": unicorn.arm_const.UC_ARM_REG_S16,
                "s17": unicorn.arm_const.UC_ARM_REG_S17,
                "d9": unicorn.arm_const.UC_ARM_REG_D9,
                "s18": unicorn.arm_const.UC_ARM_REG_S18,
                "s19": unicorn.arm_const.UC_ARM_REG_S19,
                "q5": unicorn.arm_const.UC_ARM_REG_Q5,
                "d10": unicorn.arm_const.UC_ARM_REG_D10,
                "s20": unicorn.arm_const.UC_ARM_REG_S20,
                "s21": unicorn.arm_const.UC_ARM_REG_S21,
                "d11": unicorn.arm_const.UC_ARM_REG_D11,
                "s22": unicorn.arm_const.UC_ARM_REG_S22,
                "s23": unicorn.arm_const.UC_ARM_REG_S23,
                "q6": unicorn.arm_const.UC_ARM_REG_Q6,
                "d12": unicorn.arm_const.UC_ARM_REG_D12,
                "s24": unicorn.arm_const.UC_ARM_REG_S24,
                "s25": unicorn.arm_const.UC_ARM_REG_S25,
                "d13": unicorn.arm_const.UC_ARM_REG_D13,
                "s26": unicorn.arm_const.UC_ARM_REG_S26,
                "s27": unicorn.arm_const.UC_ARM_REG_S27,
                "q7": unicorn.arm_const.UC_ARM_REG_Q7,
                "d14": unicorn.arm_const.UC_ARM_REG_D14,
                "s28": unicorn.arm_const.UC_ARM_REG_S28,
                "s29": unicorn.arm_const.UC_ARM_REG_S29,
                "d15": unicorn.arm_const.UC_ARM_REG_D15,
                "s30": unicorn.arm_const.UC_ARM_REG_S30,
                "s31": unicorn.arm_const.UC_ARM_REG_S31,
                "q8": unicorn.arm_const.UC_ARM_REG_Q8,
                "d16": unicorn.arm_const.UC_ARM_REG_D16,
                "d17": unicorn.arm_const.UC_ARM_REG_D17,
                "q9": unicorn.arm_const.UC_ARM_REG_Q9,
                "d18": unicorn.arm_const.UC_ARM_REG_D18,
                "d19": unicorn.arm_const.UC_ARM_REG_D19,
                "q10": unicorn.arm_const.UC_ARM_REG_Q10,
                "d20": unicorn.arm_const.UC_ARM_REG_D20,
                "d21": unicorn.arm_const.UC_ARM_REG_D21,
                "q11": unicorn.arm_const.UC_ARM_REG_Q11,
                "d22": unicorn.arm_const.UC_ARM_REG_D22,
                "d23": unicorn.arm_const.UC_ARM_REG_D23,
                "q12": unicorn.arm_const.UC_ARM_REG_Q12,
                "d24": unicorn.arm_const.UC_ARM_REG_D24,
                "d25": unicorn.arm_const.UC_ARM_REG_D25,
                "q13": unicorn.arm_const.UC_ARM_REG_Q13,
                "d26": unicorn.arm_const.UC_ARM_REG_D26,
                "d27": unicorn.arm_const.UC_ARM_REG_D27,
                "q14": unicorn.arm_const.UC_ARM_REG_Q14,
                "d28": unicorn.arm_const.UC_ARM_REG_D28,
                "d29": unicorn.arm_const.UC_ARM_REG_D29,
                "q15": unicorn.arm_const.UC_ARM_REG_Q15,
                "d30": unicorn.arm_const.UC_ARM_REG_D30,
                "d31": unicorn.arm_const.UC_ARM_REG_D31,
            }
        )


class ARMv5TMachineDef(ARMMachineMixinM, ARMMachineDef):
    """Unicorn machine definition for ARMv5T little-endian"""

    arch = Architecture.ARM_V5T
    byteorder = Byteorder.LITTLE


class ARMv6MMachineDef(ARMMachineMixinFP, ARMMachineMixinM, ARMMachineDef):
    """Unicorn machine definition for ARMv6-M little-endian"""

    arch = Architecture.ARM_V6M
    byteorder = Byteorder.LITTLE


class ARMv6MThumbMachineDef(ARMv6MMachineDef):
    """Unicorn machine definition for ARMv6-M little-endian, THUMB ISA"""

    arch = Architecture.ARM_V6M_THUMB
    uc_mode = unicorn.UC_MODE_THUMB


class ARMv7MMachineDef(ARMMachineMixinFP, ARMMachineMixinM, ARMMachineDef):
    """Unicorn machine definition for ARMv7-M little-endian"""

    arch = Architecture.ARM_V7M
    byteorder = Byteorder.LITTLE


class ARMv7RMachineDef(ARMMachineMixinVFP, ARMMachineMixinRA, ARMMachineDef):
    """Unicorn machine definition for ARMv7-R little-endian"""

    arch = Architecture.ARM_V7R
    byteorder = Byteorder.LITTLE


class ARMv7AMachineDef(ARMMachineMixinVFP, ARMMachineMixinRA, ARMMachineDef):
    """Unicorn machine definition for ARMv7-A little-endian"""

    arch = Architecture.ARM_V7A
    byteorder = Byteorder.LITTLE



import angr

from .bounds import BoundedExplorationMixin
from .terminate import TerminationExplorationMixin


class DefaultExplorationTechnique(
    TerminationExplorationMixin,
    BoundedExplorationMixin,
    angr.exploration_techniques.suggestions.Suggestions,
):
    """Default exploration technique.

    Registers a few default-useful plugins for the SimulationManager.
    """

    pass



from ....c99.signal import Raise, Signal
from ..systemv import MIPS64ELSysVModel


class MIPS64ELSysVRaise(Raise, MIPS64ELSysVModel):
    pass


class MIPS64ELSysVSignal(Signal, MIPS64ELSysVModel):
    pass


__all__ = [
    "MIPS64ELSysVRaise",
    "MIPS64ELSysVSignal",
]



import typing

from .state import Register


class X86MMRRegister(Register):
    """x86 Memory Management Register

    These things have a special internal format,
    represented by the 4-tuple (selector, base, limit, flags).

    Unicorn takes these as a 4-tuple.

    angr doesn't take an actual value;
    it takes a pointer to a host-allocated table.  I think.
    """

    def set_content(self, content: typing.Optional[typing.Any]):
        self._content = content

    def __str__(self):
        s = f"Reg({self.name},{self.size})="
        x = self.get_content()
        if x is None:
            s = s + "=None"
        elif isinstance(x, tuple):
            s = s + ", ".join(map(lambda v: hex(v), x))
        else:
            s = s + "External ref {hex(x)}"
        return s



import abc
import code
import logging
import pdb
import typing

from ... import emulators, exceptions, platforms, utils
from .. import state

logger = logging.getLogger(__name__)


class Hook(state.Stateful):
    """Hook a particular instruction with a function.

    Runs the function provided when the emulator reaches (but has not
    yet executed) the instruction at a particular address.

    Arguments:
        address: The address of the instruction.
        function: The function to run.
    """

    def __init__(
        self, address: int, function: typing.Callable[[emulators.Emulator], None]
    ):
        self._address = address
        self._function = function

    def extract(self, emulator: emulators.Emulator) -> None:
        # Hooks have no state to extract;
        # Just as we assume the memory layout is the same,
        # we have to assume the hook layout is the same.
        pass

    def apply(self, emulator: emulators.Emulator) -> None:
        if not isinstance(emulator, emulators.InstructionHookable):
            raise exceptions.ConfigurationError("Emulator cannot hook instructions")
        emulator.hook_instruction(self._address, self._function)


class Breakpoint(Hook):
    """An interactive breakpoint.

    Stops execution at the specified address and opens an interactive
    shell, as specified in the self.interact method.

    Arguments:
        address: The address of the breakpoint.

    """

    def __init__(self, address: int):
        super().__init__(address=address, function=self.interact)

    @staticmethod
    @abc.abstractmethod
    def interact(emulator: emulators.Emulator) -> None:
        pass


class PDBBreakpoint(Breakpoint):
    """A PDB interactive breakpoint."""

    @staticmethod
    def interact(emulator: emulators.Emulator) -> None:
        pdb.set_trace()


class PythonShellBreakpoint(Breakpoint):
    """A Python shell interactive breakpoint."""

    @staticmethod
    def interact(emulator: emulators.Emulator) -> None:
        code.interact(local={"emulator": emulator})


class Model(Hook):
    """A runtime function model implemented in Python.

    If execution reaches the given address, call the function assigned
    to self.model, instead of any code at that address in the
    emulator, and return. This is most often used to model an external
    function, e.g., libc `fread`. It is the responsibility of the
    model to read arguments and generate reasonable return values.

    Some models require static scratch space to operate.
    The quantity is stored in the 'static_space_required' attribute.
    If true, the harness must set the 'static_buffer_address' property
    of this model object.  The model will take care of mapping
    a buffer of the appropriate size at that address.

    A harness doesn't need to include a `Memory` object
    for a static buffer.  A harness can include such an object
    if it wants to initialize that memory with a specific value, or if it
    wants to inspect the contents of that buffer via `Machine.extract()`

    Arguments:
        address: The address to model.
    """

    def __init__(self, address: int):
        super().__init__(address=address, function=self.model)
        self.static_buffer_address: typing.Optional[int] = None

    @property
    @abc.abstractmethod
    def name(self) -> str:
        """A name for this model, e.g., fread or ioctl."""
        return ""

    @property
    @abc.abstractmethod
    def platform(self) -> platforms.Platform:
        """The platform for which this model is defined."""
        pass

    @property
    @abc.abstractmethod
    def abi(self) -> platforms.ABI:
        """The ABI according to which this model works."""
        pass

    static_space_required: int = 0

    @classmethod
    def lookup(
        cls, name: str, platform: platforms.Platform, abi: platforms.ABI, address: int
    ):
        """Instantiate a model by name, platform, and ABI.

        Arguments:
            name: The name of the model.
            platform: The platform for which this model is defined.
            abi: The ABI according to which this model works.
            address: The instruction address which the model will hook.

        Returns:
            The fully instantiated model.
        """
        try:
            return utils.find_subclass(
                cls,
                lambda x: x.name == name and x.platform == platform and x.abi == abi,
                address,
            )
        except ValueError:
            raise ValueError(f"no model for '{name}' on {platform} with ABI '{abi}'")

    def apply(self, emulator: emulators.Emulator) -> None:
        logger.debug(f"Hooking Model {self} {self._address:x}")

        if self.static_space_required != 0:
            # We need a static buffer.
            if self.static_buffer_address is None:
                # Harness author forgot to reserve us one
                raise exceptions.ConfigurationError(
                    f"No static buffer address provided for {self.name}"
                )
            emulator.map_memory(self.static_buffer_address, self.static_space_required)

        # Map just enough memory to jump to the model address without faulting.
        emulator.map_memory(self._address, 16)

        # Add the function hook to the emulator
        if not isinstance(emulator, emulators.FunctionHookable):
            raise exceptions.ConfigurationError("Emulator cannot hook functions")
        emulator.hook_function(self._address, self._function)

    @abc.abstractmethod
    def model(self, emulator: emulators.Emulator) -> None:
        """This is the implementation of the model for the named function.

        Note that implementation will have to make use of knowledge of
        the ABI to obtain arguments and return results, as well as
        modeling the semantics of the modeled functions appropriately.

        """
        pass


__all__ = [
    "Hook",
    "Breakpoint",
    "PDBBreakpoint",
    "PythonShellBreakpoint",
    "Model",
]



import typing

from ....platforms import Architecture, Byteorder
from .machdef import GhidraMachineDef


class XTensaMachineDef(GhidraMachineDef):
    arch = Architecture.XTENSA
    _registers: typing.Dict[str, typing.Optional[str]] = {}
    _registers |= {f"a{i}": f"a{i}" for i in range(0, 16)}
    _registers |= {"pc": "pc", "sar": "sar", "sp": "a1"}


class XTensaELMachineDef(XTensaMachineDef):
    byteorder = Byteorder.LITTLE
    language_id = "Xtensa:LE:32:default"


class XTensaBEMachineDef(XTensaMachineDef):
    byteorder = Byteorder.BIG
    language_id = "Xtensa:BE:32:default"



import typing

import capstone

from .bsid import BSIDMemoryReferenceOperand
from .instructions import Instruction, MemoryReferenceOperand, Operand, RegisterOperand


class x86Instruction(Instruction):
    word_size = 4
    angr_arch = "X86"
    cs_arch = capstone.CS_ARCH_X86
    cs_mode = capstone.CS_MODE_32
    sp = "esp"

    def _memory_reference(self, base, index, scale, offset, size):
        return BSIDMemoryReferenceOperand(
            base=base, index=index, scale=scale, offset=offset, size=size
        )

    # operand is a capstone operand
    def _memory_reference_operand(self, operand) -> MemoryReferenceOperand:
        return self._memory_reference(
            self._instruction.reg_name(operand.value.mem.base),
            self._instruction.reg_name(operand.value.mem.index),
            operand.value.mem.scale,
            operand.value.mem.disp,
            operand.size,
        )

    @property
    def reads(self) -> typing.Set[Operand]:
        """Registers and memory references read by this instruction.

        This is a list of string register names and dictionary memory reference
        specifications (i.e., in the form `base + scale * index + offset`).
        """

        reg_reads, _ = self._instruction.regs_access()
        the_reads: typing.Set[Operand] = set(
            [RegisterOperand(self._instruction.reg_name(r)) for r in reg_reads]
        )

        for operand in self._instruction.operands:
            if operand.access & capstone.CS_AC_READ:
                if operand.type == capstone.x86.X86_OP_MEM:
                    if not self._instruction.mnemonic == "lea":
                        # add the actual memory read
                        the_reads.add(self._memory_reference_operand(operand))
                    # add reads for parts of the operand
                    base_name = self._instruction.reg_name(operand.mem.base)
                    index_name = self._instruction.reg_name(operand.mem.index)
                    if base_name:
                        the_reads.add(RegisterOperand(base_name))
                    if index_name:
                        the_reads.add(RegisterOperand(index_name))
                elif operand.type == capstone.x86.X86_OP_REG:
                    # these are already in the list bc of regs_access above
                    pass
                else:
                    # shouldn't happen
                    assert 1 == 0
        if self._instruction.mnemonic == "pop":
            the_reads.add(self._memory_reference(self.sp, None, 1, 0, self.word_size))

        return the_reads

    @property
    def writes(self) -> typing.Set[Operand]:
        """Registers and memory references written by this instruction.

        Same format as `reads`.
        """
        _, reg_writes = self._instruction.regs_access()
        the_writes: typing.Set[Operand] = set(
            [RegisterOperand(self._instruction.reg_name(r)) for r in reg_writes]
        )
        for operand in self._instruction.operands:
            if operand.access & capstone.CS_AC_WRITE:
                # please dont change this to CS_OP_MEM bc that doesnt work?
                if operand.type == capstone.x86.X86_OP_MEM:
                    assert not (self._instruction.mnemonic == "lea")
                    the_writes.add(self._memory_reference_operand(operand))
                elif operand.type == capstone.x86.X86_OP_REG:
                    # operands doesn't contain implicit registers.
                    # we get all registers from regs_write
                    pass
                else:
                    assert 1 == 0
        if self._instruction.mnemonic == "push":
            the_writes.add(self._memory_reference(self.sp, None, 1, 0, self.word_size))

        return the_writes


class AMD64Instruction(x86Instruction):
    angr_arch = "AMD64"
    cs_mode = capstone.CS_MODE_64
    sp = "rsp"
    word_size = 8



import capstone

from .bsid import BSIDMemoryReferenceOperand
from .instructions import Instruction, MemoryReferenceOperand


class ARMInstruction(Instruction):
    angr_arch = "ARM"
    cs_arch = capstone.CS_ARCH_ARM
    cs_mode = capstone.CS_MODE_ARM

    def _memory_reference(self, operand) -> MemoryReferenceOperand:
        return BSIDMemoryReferenceOperand(
            base=self._instruction.reg_name(operand.value.mem.base),
            index=self._instruction.reg_name(operand.value.mem.index),
            offset=operand.value.mem.disp,
            size=4,
        )



from ..... import platforms
from .....exceptions import ConfigurationError
from ..structs import ElfRela
from .rela import ElfRelocator


class XtensaElfRelocator(ElfRelocator):
    arch = platforms.Architecture.XTENSA
    byteorder = platforms.Byteorder.LITTLE

    def _compute_value(self, rela: ElfRela):
        # Xtensa doesn't have a dynamic linker.
        raise ConfigurationError(
            "Unknown relocation type for {rela.symbol.name}: {rela.type}"
        )



from ....c99 import (
    Clearerr,
    Fclose,
    Feof,
    Ferror,
    Fflush,
    Fgetc,
    Fgetpos,
    Fgets,
    Fopen,
    Fprintf,
    Fputc,
    Fputs,
    Fread,
    Freopen,
    Fscanf,
    Fseek,
    Fsetpos,
    Ftell,
    Fwrite,
    Getc,
    Getchar,
    Gets,
    Printf,
    Putc,
    Putchar,
    Puts,
    Remove,
    Rename,
    Rewind,
    Scanf,
    Snprintf,
    Sprintf,
    Sscanf,
    Tmpfile,
    Tmpnam,
    Ungetc,
    Vfprintf,
    Vfscanf,
    Vprintf,
    Vscanf,
    Vsnprintf,
    Vsprintf,
    Vsscanf,
)
from ..systemv import PowerPCSysVModel


class PowerPCSysVFclose(Fclose, PowerPCSysVModel):
    pass


class PowerPCSysVFeof(Feof, PowerPCSysVModel):
    pass


class PowerPCSysVFerror(Ferror, PowerPCSysVModel):
    pass


class PowerPCSysVFgetc(Fgetc, PowerPCSysVModel):
    pass


class PowerPCSysVFgets(Fgets, PowerPCSysVModel):
    pass


class PowerPCSysVFopen(Fopen, PowerPCSysVModel):
    pass


class PowerPCSysVFprintf(Fprintf, PowerPCSysVModel):
    pass


class PowerPCSysVFputc(Fputc, PowerPCSysVModel):
    pass


class PowerPCSysVFputs(Fputs, PowerPCSysVModel):
    pass


class PowerPCSysVFread(Fread, PowerPCSysVModel):
    pass


class PowerPCSysVFscanf(Fscanf, PowerPCSysVModel):
    pass


class PowerPCSysVFseek(Fseek, PowerPCSysVModel):
    pass


class PowerPCSysVFtell(Ftell, PowerPCSysVModel):
    pass


class PowerPCSysVFwrite(Fwrite, PowerPCSysVModel):
    pass


class PowerPCSysVGetc(Getc, PowerPCSysVModel):
    pass


class PowerPCSysVGetchar(Getchar, PowerPCSysVModel):
    pass


class PowerPCSysVGets(Gets, PowerPCSysVModel):
    pass


class PowerPCSysVPrintf(Printf, PowerPCSysVModel):
    pass


class PowerPCSysVPutc(Putc, PowerPCSysVModel):
    pass


class PowerPCSysVPutchar(Putchar, PowerPCSysVModel):
    pass


class PowerPCSysVPuts(Puts, PowerPCSysVModel):
    pass


class PowerPCSysVRemove(Remove, PowerPCSysVModel):
    pass


class PowerPCSysVRename(Rename, PowerPCSysVModel):
    pass


class PowerPCSysVRewind(Rewind, PowerPCSysVModel):
    pass


class PowerPCSysVScanf(Scanf, PowerPCSysVModel):
    pass


class PowerPCSysVSnprintf(Snprintf, PowerPCSysVModel):
    pass


class PowerPCSysVSprintf(Sprintf, PowerPCSysVModel):
    pass


class PowerPCSysVSscanf(Sscanf, PowerPCSysVModel):
    pass


class PowerPCSysVClearerr(Clearerr, PowerPCSysVModel):
    pass


class PowerPCSysVFflush(Fflush, PowerPCSysVModel):
    pass


class PowerPCSysVFreopen(Freopen, PowerPCSysVModel):
    pass


class PowerPCSysVFgetpos(Fgetpos, PowerPCSysVModel):
    pass


class PowerPCSysVFsetpos(Fsetpos, PowerPCSysVModel):
    pass


class PowerPCSysVTmpfile(Tmpfile, PowerPCSysVModel):
    pass


class PowerPCSysVTmpnam(Tmpnam, PowerPCSysVModel):
    pass


class PowerPCSysVUngetc(Ungetc, PowerPCSysVModel):
    pass


class PowerPCSysVVfprintf(Vfprintf, PowerPCSysVModel):
    pass


class PowerPCSysVVfscanf(Vfscanf, PowerPCSysVModel):
    pass


class PowerPCSysVVprintf(Vprintf, PowerPCSysVModel):
    pass


class PowerPCSysVVscanf(Vscanf, PowerPCSysVModel):
    pass


class PowerPCSysVVsnprintf(Vsnprintf, PowerPCSysVModel):
    pass


class PowerPCSysVVsprintf(Vsprintf, PowerPCSysVModel):
    pass


class PowerPCSysVVsscanf(Vsscanf, PowerPCSysVModel):
    pass


__all__ = [
    "PowerPCSysVFclose",
    "PowerPCSysVFeof",
    "PowerPCSysVFerror",
    "PowerPCSysVFgetc",
    "PowerPCSysVFgets",
    "PowerPCSysVFopen",
    "PowerPCSysVFprintf",
    "PowerPCSysVFputc",
    "PowerPCSysVFputs",
    "PowerPCSysVFread",
    "PowerPCSysVFscanf",
    "PowerPCSysVFseek",
    "PowerPCSysVFtell",
    "PowerPCSysVFwrite",
    "PowerPCSysVGetc",
    "PowerPCSysVGetchar",
    "PowerPCSysVGets",
    "PowerPCSysVPrintf",
    "PowerPCSysVPutc",
    "PowerPCSysVPutchar",
    "PowerPCSysVPuts",
    "PowerPCSysVRemove",
    "PowerPCSysVRename",
    "PowerPCSysVRewind",
    "PowerPCSysVScanf",
    "PowerPCSysVSnprintf",
    "PowerPCSysVSprintf",
    "PowerPCSysVSscanf",
    "PowerPCSysVClearerr",
    "PowerPCSysVFflush",
    "PowerPCSysVFreopen",
    "PowerPCSysVFgetpos",
    "PowerPCSysVFsetpos",
    "PowerPCSysVTmpfile",
    "PowerPCSysVTmpnam",
    "PowerPCSysVUngetc",
    "PowerPCSysVVfprintf",
    "PowerPCSysVVfscanf",
    "PowerPCSysVVprintf",
    "PowerPCSysVVscanf",
    "PowerPCSysVVsnprintf",
    "PowerPCSysVVsprintf",
    "PowerPCSysVVsscanf",
]
